@@ -1,6 +1,8 @@
 import enum
 from typing import Union, List, Dict, Any
 
+INVALID : int = -1
+
 JSON_SERIALIZABLE_TYPES = Union[str, bool, int, float, None, List[Any], Dict[Any, Any]]
 
 class LogLevel(enum.Enum):
@@ -21,3 +23,6 @@ class TrendDirection(enum.Enum):
 
     def to_string(self) -> str:
         return self.name.lower() if self != TrendDirection.UNDEFINED else ''
+
+OrderSide = enum.Enum('OrderSide', 'UNDEFINED BUY SELL')
+PositionStatus = enum.Enum("PositionStatus", 'UNDEFINED OPEN CLOSED SL')
\ No newline at end of file
