@@ -254,7 +254,11 @@ async def main() -> None:
             processed_messages = sorted(processed_messages, key=lambda m: m['datetime'])
             last_message_date = processed_messages[-1]['datetime']
             
-    redis_client: Optional[StrictRedis] = init_redis_client()
+    try:        
+        redis_client: Optional[StrictRedis] = init_redis_client()
+    except Exception as redis_err:
+        redis_client = None
+        log(f"Failed to connect to redis. Still run but not publishing to it. {redis_err}")
         
     start_date: Optional[datetime] = None
     if param.get('start_date'):
@@ -362,7 +366,7 @@ async def main() -> None:
                         processed_messages.append(message_data)
 
                         if message_date>tm1:
-                            dispatch_notification(title=f"{param['current_filename']} {param['channel_username']} started", message=message_data, footer=param['notification']['footer'], params=notification_params, log_level=LogLevel.CRITICAL, logger=logger) # type: ignore
+                            dispatch_notification(title=f"{param['current_filename']} {param['channel_username']} Incoming!", message=message_data, footer=param['notification']['footer'], params=notification_params, log_level=LogLevel.CRITICAL, logger=logger) # type: ignore
 
                             with open(message_cache_file, 'a', encoding='utf-8') as f:
                                 json.dump(message_data, f, ensure_ascii=False)
