@@ -1,3 +1,4 @@
+# type: ignore
 import sys
 import traceback
 import os
@@ -19,15 +20,15 @@ from re import Pattern
 from redis import StrictRedis
 import asyncio
 
-from util.aws_util import AwsKmsUtil
-
 import ccxt.pro as ccxtpro
 
+from siglab_py.util.retry_util import retry
+from siglab_py.util.aws_util import AwsKmsUtil
 from siglab_py.exchanges.any_exchange import AnyExchange
-from util.market_data_util import async_instantiate_exchange
+from siglab_py.util.market_data_util import async_instantiate_exchange
 from siglab_py.ordergateway.client import Order, DivisiblePosition
-from siglab_py.constants import LogLevel # type: ignore
-from util.notification_util import dispatch_notification
+from siglab_py.constants import LogLevel
+from siglab_py.util.notification_util import dispatch_notification
 
 current_filename = os.path.basename(__file__)
 
@@ -49,6 +50,7 @@ Usage:
 	--passphrase is optional, this depends on the exchange.
     --gateway_id contains two parts separated by underscore. Gateway.py will parse 'hyperliquid_01' into two parts: 'hyperliquid' (Exchange name) and '01' (Use this for your sub account ID). Exchange name need be spelt exactly. Please have a look at market_data_util async_instantiate_exchange.
     --slack_info_url, --slack_critical_url and --slack_alert_url are if you want gateway to dispatch Slack notification on events.
+    --order_amount_randomize_max_pct adds small variance to sliced order amount (Default max 10% on sliced amount) to cover your track in order executions, this is useful especially when executing bigger orders during quieter hours.
     How to get Slack webhook urls? https://medium.com/@natalia_assad/how-send-a-table-to-slack-using-python-d1a20b08abe0
 
     Another example:
@@ -212,6 +214,7 @@ param : Dict = {
     "executions_publish_topic" : r"ordergateway_executions_$GATEWAY_ID$",
 
     "default_fees_ccy" : None,
+    "order_amount_randomize_max_pct" : 0,
     "loop_freq_ms" : 500, # reduce this if you need trade faster
     "loops_random_delay_multiplier" : 1, # Add randomness to time between slices are sent off. Set to 1 if no random delay needed.
     "wait_fill_threshold_ms" : 5000,
@@ -293,7 +296,7 @@ def log(message : str, log_level : LogLevel = LogLevel.INFO):
         logger.error(f"{datetime.now()} {message}")
 
 def parse_args():
-    parser = argparse.ArgumentParser() # type: ignore
+    parser = argparse.ArgumentParser()
 
     parser.add_argument("--gateway_id", help="gateway_id: Where are you sending your order?", default=None)
     parser.add_argument("--dry_run", help="Y or N (default, for testing). If Y, orders won't be dispatched to exchange, gateway will fake reply.", default='N')
@@ -301,6 +304,7 @@ def parse_args():
     parser.add_argument("--rate_limit_ms", help="rate_limit_ms: Check your exchange rules", default=100)
 
     parser.add_argument("--default_fees_ccy", help="If you're trading crypto, CEX fees USDT, DEX fees USDC in many cases. Default None, in which case gateway won't aggregatge fees from executions for you.", default=None)
+    parser.add_argument("--order_amount_randomize_max_pct", help="Randomize order amount to hide your track? This is max percentage variance applied on sliced amount (not entire order amount)", default=10)
     parser.add_argument("--loop_freq_ms", help="Loop delays. Reduce this if you want to trade faster.", default=500)
     parser.add_argument("--wait_fill_threshold_ms", help="Wait for fills for how long?", default=5000)
 
@@ -328,6 +332,7 @@ def parse_args():
     param['default_type'] = args.default_type
     param['rate_limit_ms'] = int(args.rate_limit_ms)
     param['default_fees_ccy'] = args.default_fees_ccy
+    param['order_amount_randomize_max_pct'] = float(args.order_amount_randomize_max_pct)
     param['loop_freq_ms'] = int(args.loop_freq_ms)
     param['wait_fill_threshold_ms'] = int(args.wait_fill_threshold_ms)
 
@@ -371,7 +376,7 @@ async def watch_orders_task(
 ):
     while True:
         try:
-            order_updates = await exchange.watch_orders() # type: ignore
+            order_updates = await exchange.watch_orders()
             for order_update in order_updates:
                 order_id = order_update['id']
                 executions[order_id] = order_update
@@ -390,11 +395,15 @@ async def send_heartbeat(exchange):
         try:
             first_ws_url = next(iter(exchange.clients))
             client = exchange.clients[first_ws_url]
-            message = exchange.ping(client)
-            await client.send(message)
-            log('Heartbeat sent')
+            if exchange.ping:
+                message = exchange.ping(client)
+                await client.send(message)
+                log('Heartbeat sent')
+            else:
+                log(f'Please check https://github.com/ccxt/ccxt/blob/master/python/ccxt/pro/{exchange.name} if ping was implemented')
+            
         except Exception as hb_error:
-            log(f'Failed to send heartbeat: {hb_error}')
+            log(f'Failed to send heartbeat: {hb_error} {str(sys.exc_info()[0])} {str(sys.exc_info()[1])} {traceback.format_exc()}')
         finally:
             await asyncio.sleep(20)
 
@@ -406,15 +415,17 @@ async def execute_one_position(
     notification_params : Dict[str, Any]
 ):
     try:
-        market : Dict[str, Any] = exchange.markets[position.ticker] if position.ticker in exchange.markets else None # type: ignore
+        market : Dict[str, Any] = exchange.markets[position.ticker] if position.ticker in exchange.markets else None
         if not market:
-            raise ArgumentError(f"Market not found for {position.ticker} under {exchange.name}") # type: ignore
+            raise ArgumentError(f"Market not found for {position.ticker} under {exchange.name}")
 
-        min_amount = float(market['limits']['amount']['min']) if market['limits']['amount']['min'] else 0 # type: ignore
+        min_amount = float(market['limits']['amount']['min']) if market['limits']['amount']['min'] else 0
         multiplier = market['contractSize'] if 'contractSize' in market and market['contractSize'] else 1
         position.multiplier = multiplier
 
-        log(f"{position.ticker} min_amount: {min_amount}, multiplier: {multiplier}") 
+        order_amount_randomize_max_pct : float = param['order_amount_randomize_max_pct']
+
+        log(f"{position.ticker} min_amount: {min_amount}, multiplier: {multiplier}, order_amount_randomize_max_pct: {order_amount_randomize_max_pct}") 
 
         slices : List[Order] = position.to_slices()
 
@@ -428,7 +439,19 @@ async def execute_one_position(
                 slices[-1].amount += last_slice.amount
 
                 log(f"{position.ticker} Last slice residual smaller than min_amount. Amount is added to prev slice instead. last_slice_amount: {last_slice.amount/multiplier}, last_slice_rounded_amount: {last_slice_rounded_amount_in_base_ccy}") 
-
+        
+        @retry(num_attempts=3, pause_between_retries_ms=3000)
+        async def _fetch_order(
+            order_id : str, 
+            ticker : str,
+            exchange : AnyExchange,
+        ):
+            order_update = await exchange.fetch_order(order_id, ticker)
+            return order_update
+        
+        randomized_order_amount : float = 0
+        last_randomized_order_amount : float = 0
+        apply_last_randomized_amount : bool = False # False: Apply new variance, True: Apply -1 * last_randomized_order_amount
         i = 0
         for slice in slices:
             try:
@@ -436,9 +459,21 @@ async def execute_one_position(
 
                 dt_now : datetime = datetime.now()
 
-                slice_amount_in_base_ccy : float = slice.amount
+                if len(slices)>1:
+                    if not apply_last_randomized_amount and i<len(slices):
+                        randomized_order_amount = slice.amount * (order_amount_randomize_max_pct * random.uniform(-1, 1)) /100
+                        last_randomized_order_amount = randomized_order_amount
+                        
+                    else:
+                        randomized_order_amount = -1 * last_randomized_order_amount # Apply the opposite of last slice's variance
+                        last_randomized_order_amount = 0 # If # slices == 5, last slice don't apply random amount, so reset last_randomized_order_amount to zero.
+                
+                slice_amount_in_base_ccy : float = slice.amount + randomized_order_amount
+
+                apply_last_randomized_amount = not apply_last_randomized_amount
+                    
                 rounded_slice_amount_in_base_ccy = slice_amount_in_base_ccy / multiplier # After divided by multiplier, rounded_slice_amount_in_base_ccy in number of contracts actually (Not in base ccy).
-                rounded_slice_amount_in_base_ccy = exchange.amount_to_precision(position.ticker, rounded_slice_amount_in_base_ccy) # type: ignore
+                rounded_slice_amount_in_base_ccy = exchange.amount_to_precision(position.ticker, rounded_slice_amount_in_base_ccy)
                 rounded_slice_amount_in_base_ccy = float(rounded_slice_amount_in_base_ccy) if rounded_slice_amount_in_base_ccy else 0
                 rounded_slice_amount_in_base_ccy = rounded_slice_amount_in_base_ccy if rounded_slice_amount_in_base_ccy>min_amount else min_amount
 
@@ -446,7 +481,7 @@ async def execute_one_position(
                     log(f"{position.ticker} Slice amount rounded to zero?! slice amount before rounding: {slice.amount}") 
                     continue
 
-                orderbook = await exchange.fetch_order_book(symbol=position.ticker, limit=3) # type: ignore
+                orderbook = await exchange.fetch_order_book(symbol=position.ticker, limit=3)
                 if position.side=='buy':
                     asks = [ ask[0] for ask in orderbook['asks'] ]
                     best_asks = min(asks)
@@ -456,7 +491,7 @@ async def execute_one_position(
                     best_bid = max(bids)
                     limit_price : float = best_bid * (1 - position.leg_room_bps/10000)
                     
-                rounded_limit_price : float = exchange.price_to_precision(position.ticker, limit_price) # type: ignore
+                rounded_limit_price : float = exchange.price_to_precision(position.ticker, limit_price)
                 rounded_limit_price = float(rounded_limit_price)
                 
                 order_params = {
@@ -472,7 +507,7 @@ async def execute_one_position(
                             )
 
                     if not param['dry_run']:
-                        executed_order = await exchange.create_order( # type: ignore
+                        executed_order = await exchange.create_order(
                             symbol = position.ticker,
                             type = position.order_type,
                             amount = rounded_slice_amount_in_base_ccy,
@@ -499,7 +534,7 @@ async def execute_one_position(
 
                 else:
                     if not param['dry_run']:
-                        executed_order = await exchange.create_order( # type: ignore
+                        executed_order = await exchange.create_order(
                             symbol = position.ticker,
                             type = position.order_type,
                             amount = rounded_slice_amount_in_base_ccy,
@@ -596,11 +631,10 @@ async def execute_one_position(
                         elapsed_sec = time.time() - start_time
                         log(f"{position.ticker} waiting for order update ... elapsed_sec: {elapsed_sec}")
                 
-                
                 # Cancel hung limit order, resend as market
                 if order_status!='closed':
                     # If no update from websocket, do one last fetch via REST
-                    order_update = await exchange.fetch_order(order_id, position.ticker) # type: ignore 
+                    order_update = await _fetch_order(order_id, position.ticker, exchange) 
                     order_status = order_update['status']
                     filled_amount = order_update['filled']
                     remaining_amount = order_update['remaining']
@@ -610,15 +644,15 @@ async def execute_one_position(
 
                     if order_status!='closed':
                         log(f"Final order_update before cancel+resend: {json.dumps(order_update, indent=4)}", log_level=LogLevel.INFO)
-                        await exchange.cancel_order(order_id, position.ticker)  # type: ignore
+                        await exchange.cancel_order(order_id, position.ticker)
                         position.get_execution(order_id)['status'] = 'canceled'
                         log(f"Canceled unfilled/partial filled order: {order_id}. Resending remaining_amount: {remaining_amount} as market order.", log_level=LogLevel.INFO)
                         
-                        rounded_slice_amount_in_base_ccy = exchange.amount_to_precision(position.ticker, remaining_amount) # type: ignore
+                        rounded_slice_amount_in_base_ccy = exchange.amount_to_precision(position.ticker, remaining_amount)
                         rounded_slice_amount_in_base_ccy = float(rounded_slice_amount_in_base_ccy)
                         rounded_slice_amount_in_base_ccy = rounded_slice_amount_in_base_ccy if rounded_slice_amount_in_base_ccy>min_amount else min_amount
                         if rounded_slice_amount_in_base_ccy>0:
-                            executed_resent_order = await exchange.create_order(  # type: ignore
+                            executed_resent_order = await exchange.create_order(
                                 symbol=position.ticker,
                                 type='market',
                                 amount=remaining_amount,
@@ -654,7 +688,7 @@ async def execute_one_position(
 
                             if (not order_status or order_status!='closed'):
                                 # If no update from websocket, do one last fetch via REST
-                                order_update = await exchange.fetch_order(order_id, position.ticker) # type: ignore 
+                                order_update = await _fetch_order(order_id, position.ticker, exchange) 
                                 order_status = order_update['status']
                                 filled_amount = order_update['filled']
                                 remaining_amount = order_update['remaining']
@@ -691,13 +725,16 @@ async def execute_one_position(
         for dispatched_slice in position.dispatched_slices:
             log(f"{json.dumps(dispatched_slice.to_dict(), indent=4)}")
 
+        position_from_exchange = await exchange.fetch_position(position.ticker)
+        log(f"{json.dumps(position_from_exchange.to_dict(), indent=4)}")
+
         position.filled_amount = position.get_filled_amount()
         position.average_cost = position.get_average_cost()
         position.fees = position.get_fees()
 
-        balances = await exchange.fetch_balance() # type: ignore
+        balances = await exchange.fetch_balance()
         if param['default_type']!='spot':
-            updated_position = await exchange.fetch_position(symbol=position.ticker) # type: ignore
+            updated_position = await exchange.fetch_position(symbol=position.ticker)
             # After position closed, 'updated_position' can be an empty dict. hyperliquid for example.
             amount = (updated_position['contracts'] if updated_position else 0) * position.multiplier # in base ccy
         else:
@@ -725,7 +762,7 @@ async def execute_one_position(
         err_msg = f"{position.ticker}  Execution failed: {position_execution_err} {str(sys.exc_info()[0])} {str(sys.exc_info()[1])} {traceback.format_exc()}"
         log(err_msg)
 
-        dispatch_notification(title=f"{param['current_filename']} {param['gateway_id']} {position.ticker} execute_one_position failed!!!", message=err_msg, footer=param['notification']['footer'], params=notification_params, log_level=LogLevel.ERROR, logger=logger) # type: ignore
+        dispatch_notification(title=f"{param['current_filename']} {param['gateway_id']} {position.ticker} execute_one_position failed!!!", message=err_msg, footer=param['notification']['footer'], params=notification_params, log_level=LogLevel.ERROR, logger=logger)
 
         position.done = False
         position.execution_err = err_msg
@@ -866,9 +903,9 @@ async def main():
     )
     if exchange:
         # Once exchange instantiated, try fetch_balance to confirm connectivity and test credentials.
-        balances = await exchange.fetch_balance() # type: ignore
+        balances = await exchange.fetch_balance()
         log(f"{param['gateway_id']}: account balances {balances}")
-        dispatch_notification(title=f"{param['current_filename']} {param['gateway_id']} started", message=balances, footer=param['notification']['footer'], params=notification_params, log_level=LogLevel.CRITICAL, logger=logger)
+        dispatch_notification(title=f"{param['current_filename']} {param['gateway_id']} started", message=balances['total'], footer=param['notification']['footer'], params=notification_params, log_level=LogLevel.CRITICAL, logger=logger)
 
         await work(param=param, exchange=exchange, redis_client=redis_client, notification_params=notification_params)
 
