@@ -47,7 +47,7 @@ class AnalyticUtilTests(unittest.TestCase):
             'datetime', 'datetime_utc', 'year', 'month', 'day', 'hour', 'minute', 'dayofweek', 
             'pct_chg_on_close', 'candle_height', 'candle_body_height',
             'week_of_month', 'apac_trading_hr', 'emea_trading_hr', 'amer_trading_hr', 
-            'is_green', 'pct_change_close', 
+            'is_green', 'candle_class', 'pct_change_close', 
             'sma_short_periods', 'sma_long_periods', 'ema_short_periods', 'ema_long_periods', 'ema_close', 
             'std', 'std_percent', 
             'vwap_short_periods', 'vwap_long_periods',
@@ -70,7 +70,7 @@ class AnalyticUtilTests(unittest.TestCase):
             'rsi', 'rsi_bucket', 'ema_rsi', 'rsi_max', 'rsi_idmax', 'rsi_min', 'rsi_idmin', 'rsi_trend', 'rsi_trend_from_highs', 'rsi_trend_from_lows', 'rsi_divergence',
             'typical_price', 
             'money_flow', 'money_flow_positive', 'money_flow_negative', 'positive_flow_sum', 'negative_flow_sum', 'money_flow_ratio', 'mfi', 'mfi_bucket',
-            'macd', 'signal', 'macd_minus_signal', 
+            'macd', 'signal', 'macd_minus_signal', 'macd_cross', 'macd_bullish_cross_last_id', 'macd_bearish_cross_last_id', 'macd_cross_last',
             'fib_0.618_short_periods', 'fib_0.618_long_periods', 
             'gap_close_vs_ema', 'gap_close_vs_ema_percent',
             'close_above_or_below_ema', 
