@@ -12,7 +12,6 @@ import asyncio
 from tabulate import tabulate
 
 from ccxt.base.exchange import Exchange as CcxtExchange
-from ccxt import deribit
 import ccxt
 import ccxt.pro as ccxtpro
 
@@ -25,6 +24,7 @@ import yfinance as yf
 from siglab_py.util.retry_util import retry
 from siglab_py.exchanges.futubull import Futubull
 from siglab_py.exchanges.any_exchange import AnyExchange
+from siglab_py.exchanges.deribit import Deribit, DeribitAsync
 
 def instantiate_exchange(
     exchange_name : str,
@@ -61,7 +61,7 @@ def instantiate_exchange(
     elif exchange_name=='okx':
         exchange = ccxt.okx(exchange_params) # type: ignore
     elif exchange_name=='deribit':
-        exchange = ccxt.deribit(exchange_params)  # type: ignore
+        exchange = Deribit(exchange_params)  # type: ignore
     elif exchange_name=='hyperliquid':
         exchange = ccxt.hyperliquid(
             {
@@ -120,7 +120,7 @@ async def async_instantiate_exchange(
     elif exchange_name=='deribit':
         # spot, swap, future
         # https://github.com/ccxt/ccxt/blob/master/python/ccxt/deribit.py#L360
-        exchange = ccxtpro.deribit(exchange_params)  # type: ignore
+        exchange = DeribitAsync(exchange_params)  # type: ignore
     elif exchange_name=='kraken':
         exchange = ccxtpro.kraken(exchange_params) # type: ignore
     elif exchange_name=='hyperliquid':
@@ -297,6 +297,17 @@ def fix_column_types(pd_candles : pd.DataFrame):
     pd_candles.reset_index(drop=True, inplace=True)
     pd_candles.sort_values("datetime", inplace=True)
 
+def interval_to_ms(interval : str) -> int:
+    interval_ms : int = 0
+    if interval=="d":
+        interval_ms = 24*60*60*1000
+    elif interval=="h":
+        interval_ms = 60*60*1000
+    elif interval=="m":
+        interval_ms = 60*1000
+
+    return interval_ms
+
 '''
 https://polygon.io/docs/stocks
 '''
@@ -668,12 +679,13 @@ def _fetch_candles_ccxt(
         A more efficient way is to find listing date. Start looping from there.
         '''
         market = exchange.markets[ticker]
+        this_ticker_start_ts = start_ts
         if market['created']:
-            start_ts = max(start_ts, int(market['created']/1000))
+            this_ticker_start_ts = max(this_ticker_start_ts, int(market['created']/1000))
 
         all_candles = []
         params = {}
-        this_cutoff = start_ts
+        this_cutoff = this_ticker_start_ts
         while this_cutoff<end_ts:
             candles = _fetch_ohlcv(exchange=exchange, symbol=ticker, timeframe=candle_size, since=int(this_cutoff * 1000), limit=num_candles_limit, params=params)
             if candles and len(candles)>0:
@@ -707,7 +719,7 @@ def fetch_deribit_btc_option_expiries(
         Dict[str, Dict[str, Union[str, float]]]
     ]
 ]:
-    exchange = deribit()
+    exchange = ccxt.deribit()
     instruments = exchange.public_get_get_instruments({
         'currency': market,
         'kind': 'option',
@@ -763,7 +775,9 @@ def fetch_deribit_btc_option_expiries(
 
 def build_pair_candles(
     pd_candles1 : pd.DataFrame,
-    pd_candles2 : pd.DataFrame
+    pd_candles2 : pd.DataFrame,
+    left_columns_postfix : str = "_1",
+    right_columns_postfix : str = "_2"
 ) -> pd.DataFrame:
     min_timestamp_ms1 = int(pd_candles1.iloc[0]['timestamp_ms'])
     max_timestamp_ms1 = int(pd_candles1.iloc[-1]['timestamp_ms'])
@@ -793,16 +807,16 @@ def build_pair_candles(
     assert(max_timestamp_ms1==max_timestamp_ms2)
     assert(pd_candles1.shape[0]==pd_candles2.shape[0])
 
-    if len([ col for col in pd_candles1.columns if col[-2:]=='_1' ]) == 0:
-        pd_candles1.columns = [str(col) + '_1' for col in pd_candles1.columns]
+    if len([ col for col in pd_candles1.columns if col[-2:]==left_columns_postfix ]) == 0:
+        pd_candles1.columns = [str(col) + left_columns_postfix for col in pd_candles1.columns]
 
-    if len([ col for col in pd_candles2.columns if col[-2:]=='_2' ]) == 0:
-        pd_candles2.columns = [str(col) + '_2' for col in pd_candles2.columns]
+    if len([ col for col in pd_candles2.columns if col[-2:]==right_columns_postfix ]) == 0:
+        pd_candles2.columns = [str(col) + right_columns_postfix for col in pd_candles2.columns]
 
     pd_candles1.reset_index(drop=True, inplace=True)
     pd_candles2.reset_index(drop=True, inplace=True)
     pd_candles = pd.concat([pd_candles1, pd_candles2], axis=1)
-    pd_candles['timestamp_ms_gap'] = pd_candles['timestamp_ms_1'] - pd_candles['timestamp_ms_2']
+    pd_candles['timestamp_ms_gap'] = pd_candles[f'timestamp_ms{left_columns_postfix}'] - pd_candles[f'timestamp_ms{right_columns_postfix}']
     assert(pd_candles[pd_candles.timestamp_ms_gap!=0].shape[0]==0)
 
     pd_candles.drop(pd_candles.columns[pd_candles.columns.str.contains('unnamed',case = False)],axis = 1, inplace = True)
